%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftcigen                          ftcigen.m                 ftcigen
% =========================================================================  
%                          FlexTools(CI) for Matlab
%                             Copyright (c) 1996
%                       Flexible Intelligence Group (FIG)                            
%                   P. O. Box 861477, Tuscaloosa, AL 35486-0013, USA.            
%                                 06/05/1996                                  
% =========================================================================  
% =========================================================================   
%  PURPOSE   
%         Do certain initialization operations for CI
%  .........................................................................   
%  GIVEN  
%		
%  .........................................................................   
%  OPERATION   
%
%  		 Declare Variables
%  .........................................................................   
%  RETURNS   
%         Nothing. Global variables gets set up.
%  .........................................................................   
%  APPLICATION   
%         CI, ENM, EFM
%  .........................................................................   
%  USE WITH   
%   
%       MUST   
%               FlexTool(CI) FlexTools(ENM), FlexTools(EFM) Matlab   
%       MAY   
%                
%  USES
%  ========================================================================   
%   SYNTAX  /  USAGE  
%   ftcigen
%  ========================================================================  

%  DECLARATION   
%   ftcigen
   
%  ========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%
%   
%  ========================================================================   
%  VARIABLE DICTIONARY   
%
%  ========================================================================   
%  ALGORITHM   
%   
%  ========================================================================   
%  CODE
%
% GLOBAL

%
global ftMaxModelCNTgbl  ftModelCNTgbl    ftModelIDXgbl
global ftModelNameMATgbl ftModelDBVECgbl  ftModelPartCntVECgbl
global ftMaxPartCNTgbl   ftPartTypeVECgbl ftPartCNTgbl
global ftPartIDXgbl      ftPartNameMATgbl ftPartDBVECgbl 
global ftModelTypeVECgbl ftSequenceVECgbl ftEApartCNTgbl
global ftPartInfoMATgbl  ftParmCNTgbl     ftParmIDXgbl
%
global ftParmUserCNTgbl  ftParmUserMATgbl ftParmUserNameMATgbl
global ftPartUserCNTgbl
%
global ftUserInfoMATgbl   ftParmUserMATgbl   
global ftParmUserFLGgbl   ftParmUserVECgbl
global ftInputScaleVECgbl ftOutputScaleVECgbl
global ftIScaleValMATgbl  ftOScaleValMATgbl 
global ftIScaleFLGgbl     ftOScaleFLGgbl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%
% MODEL
%%%%%%%%

% Maximum number of models allowed.
ftMaxModelCNTgbl = 20;

% Current number of Models
ftModelCNTgbl=0;

% Current model index
ftModelIDXgbl=0;

% Model Type Name Matrix
%  
% Expand this matrix....
ftModelNameMATgbl = str2mat('Fuzzy Controller',...
			    'TSK Fuzzy Controller',...
			    'Fuzzy Decision Maker',...
			    'Neural Network');

% Model Type Database vector (code No. for FS, NN, etc.)
%
% Number of entries depends upon the type of the model
% supported.
ftModelDBVECgbl = [1 2 3 4];

% Selected Model Type vector (example: [1 4 ...] for 
% Fuzzy Controller, and Neural Network
% 
ftModelTypeVECgbl = zeros(1,ftMaxModelCNTgbl);

% Specifies the sequence of learning of the models
% example: [ 1 1 2 1 2 ] identifies 5 models, Models
%          1,2,4 learn together, then 3,5.
ftSequenceVECgbl  = zeros(1,ftMaxModelCNTgbl);

%
% Keeps track of the starting Partition index for each
% Model. example: [1 4] : partition of model 1 start  at 1
%                         partition of model 2 starts at 4
%
ftModelPartFromVECgbl = ftModelTypeVECgbl;

%
% specifies the number of partitions for each model
% The column number represents the model number.
%
ftModelPartCntVECgbl     = zeros(1,ftMaxModelCNTgbl);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%
% Partition
%%%%%%%%%%%

% Maximum number of partitions (there could be
% several partitions in a model!
ftMaxPartCNTgbl   = 500;

% Partition Type vector (code No. for FS, NN, etc.)
% 
ftPartTypeVECgbl = zeros(1,ftMaxPartCNTgbl);

% Current number of partitions for learning..
ftPartCNTgbl=0;

% Current partition index
ftPartIDXgbl=0;

% Partition Type Name Matrix
%  
% Expand this matrix....
ftPartNameMATgbl = str2mat('Triangular FMF',...
			    'Trapezoidal FMF',...
			    'Gaussian FMF',...
			    'NN Weights',...
			    'Rule Structure',...
			    'FMF Parameters',...
			    'No. of Rules',...
			    'Input Scaling',...
			    'Output Scaling',...
			    'FMF Tuning');
ftPartNameMATgbl = str2mat(ftPartNameMATgbl,...
			    'Input Rule Structure',...
			    'Output Rule Structure',...
			    'Input FMF Parameters',...
			    'Output FMF Parameters',...
			    'TSK Parameters',...
			    'User Specified',...
			    'NN Connections',...
			    'NN best Input Select',...
			    'RBF NN Learn SD');
			  
% Partition Type Database vector (code no. for above items)
%
% Number of entries depends upon the type of the model
% supported. One strategy could be to to use the number
% of parameters.
ftPartDBVECgbl = [1:50];

%  a 6*ftMaxPartCNTgbl matrix 
%
%  --> partition     # 1       |    # 2   | # 3     
%  ----------|------------------
%  Model Idx |   1 (Fuzzy Controller)
%  ----------|------------------
%  PartitionIdx| 4 (NN Weights)
%  -----------------------------
%  ParmType  |   1 (integer)
%  -----------------------------
%  Start     |   1
%  ----------|------------------
%  Stop      |   10
%  ----------|------------------
%  Cnt       |   10
%  ----------|------------------
ftPartInfoMATgbl=zeros(6,ftMaxPartCNTgbl);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%
% Parameters
%%%%%%%%%%%%

ftParmMaxCNTgbl=1000;

ftParmCNTgbl    = 0;
ftParmIDXgbl    = 0;

ftParmTypeNameMATgbl= str2mat('Integer',...
			      'Real',...
			      'Continuous');
ftParmTypeVECgbl= [1 2 3];

%%%%%%%%
% USERS
%%%%%%%%

ftParmUserCNTgbl	=0;
ftPartUserCNTgbl	=0;

%  a 5*ftMaxPartCNTgbl matrix 
%
%  --> partition     # 1       |    # 2   | # 3     
%  ----------|------------------
%  PartitionIdx| 4 (NN Weights)
%  -----------------------------
%  ParmType  |   1 (integer)
%  -----------------------------
%  Start     |   1
%  ----------|------------------
%  Stop      |   10
%  ----------|------------------
%  Cnt       |   10
%  ----------|------------------
ftUserInfoMATgbl        =zeros(5,ftMaxPartCNTgbl);

% 3*ftParmMaxCNTgbl matrix.
% 1: from (start)
% 2: to   (end)
% 3: parmType
%
ftParmUserMATgbl	=zeros(3,ftParmMaxCNTgbl);

ftParmUserVECgbl	=zeros(1,ftParmMaxCNTgbl);
ftParmUserNameMATgbl	=0;
ftParmUserFLGgbl        =0;

%
% For Input and Output Scaling
%

ftInputScaleVECgbl      =0;
ftOutputScaleVECgbl     =0;
ftIScaleValMATgbl        = zeros(3,ftParmMaxCNTgbl);
ftOScaleValMATgbl        = zeros(3,ftParmMaxCNTgbl);
ftIScaleFLGgbl		=0;
ftOScaleFLGgbl		=0;

%====================================================================

%  ========================================================================   
%  CAUTION   
%  ========================================================================   
%  EXAMPLE   
   
%  ========================================================================   
%  SEE ALSO   
%   
%  ========================================================================   
%  REFERENCE   
%    
%  Guidelines for Evolutionary Fuzzy Modeling
%  FIG LLC, Tuscaloosa, AL 35486-1477, USA, FIGLLC@AOL.COM
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftcigen.m
% ftcigen                           ftcigen.m                  ftcigen
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


