%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftefmeld                          ftefmeld.m                 ftefmeld
% =========================================================================  
%                          FlexTools(EFM) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)                            
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 06/05/1995                                  
% =========================================================================  
% =========================================================================   
%  PURPOSE   
%         Load the fuzzy ftFSrule_ (antecedents and consequents linguistic
%         variable matrix) and membership function parameter matrix.
%  .........................................................................   
%  GIVEN  
%		  GA optimized fuzzy parameters 
%  .........................................................................   
%  OPERATION   
%
%  		  Load parameters in appropriate matrices
%  .........................................................................   
%  RETURNS   
%         Nothing. Global matrices get loaded.
%  .........................................................................   
%  APPLICATION   
%         Fuzzy Systems, Evolutionary Fuzzy Modeling
%  .........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%  ========================================================================   
%   SYNTAX  /  USAGE  
%   ftefmeld(params)
%  ========================================================================  

%  DECLARATION   

function ftefmeld(params);
   
%  ========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%
%   
%  ========================================================================   
%  VARIABLE DICTIONARY   
%
%  ========================================================================   
%  ALGORITHM   
%   
%10	Load the matrices with the ftFSrule_ structure, and membership function
%	parameters.
%20	Compute the membership of the input values in the antecedent fuzzy 
%	subsets (using appropriate fuzzy conjunction or disjunction).
%  ========================================================================   
%  CODE
%
% GLOBAL

global ftFSncountCNTgbl ftFSbasisMATgbl xsave 
global ftFSNo_rulesCNTgbl ftFSaNo_critCNTgbl ftFSaNo_lingCNTgbl No_pnam  ftFSNo_stepsCNTgbl ftFSmaxsVECgbl ftFSminsVECgbl No_action
global ftFSaincMATgbl ftFSainlMATgbl ftFSainrMATgbl ftFSaoutcMATgbl ftFSaoutlMATgbl ftFSaoutrMATgbl ftFSparmMATgbl
global ftFSinlMATgbl ftFSoutlMATgbl ftFSiparmMATgbl ftFSoparmMATgbl ftFSmax_LVSCAgbl
global ftFSfmfSTRgbl No_fmfparm ftFSaggrparmMATgbl ftFSdefparmMATgbl  
global ftFSsum_lingVECgbl ftioLVtypeVECgbl ling_idx
global ftGApmaxVECgbl ftGApminVECgbl ftGApresVECgbl ftnopVECgbl ftGAnopSCAgbl ftEFMparmIDXgbl
global ftEFMcilrnIDXgbl ftEFMrilrnIDXgbl ftEFMcolrnIDXgbl ftEFMrolrnIDXgbl ftEFMilrnrsCNTgbl ftEFMolrnrsCNTgbl ftEFMtotlrnrsCNTgbl
global veciidx vecoidx ftEFMlrnoptSCAgbl
global ftEFMcinrsCNTgbl ftEFMcoutrsCNTgbl ftEFMrinrsCNTgbl ftEFMroutrsCNTgbl
global inlrnfmfcnt ftEFMlrnfmfCNTgbl ftEFMtotlrnCNTgbl inlrnlvcnt ftEFMlrnlvCNTgbl ftFSfmfcntVECgbl
global ftFSNo_inCNTgbl ftFSNo_outCNTgbl ftFSfmfilenMATgbl ftFSfuztypeSCAgbl ftFSrulelearnVECgbl
global vecimap vecomap
global ftFSbase_minSCAgbl ftFSbase_maxSCAgbl ftFSbase_diffSCAgbl ftFSbase_setVECgbl
global FTdel_fmfparm

global ftMaxModelCNTgbl  ftModelCNTgbl    ftModelIDXgbl
global ftModelNameMATgbl ftModelDBVECgbl  ftModelPartCntVECgbl
global ftMaxPartCNTgbl   ftPartTypeVECgbl ftPartCNTgbl
global ftPartIDXgbl      ftPartNameMATgbl ftPartDBVECgbl 
global ftModelTypeVECgbl ftSequenceVECgbl ftEApartCNTgbl
global ftPartInfoMATgbl  ftParmCNTgbl     ftParmIDXgbl
global ftParmUserCNTgbl  ftParmUserMATgbl ftParmUserNameMATgbl
global ftPartUserCNTgbl  ftUserInfoMATgbl ftParmUserMATgbl   
global ftParmUserFLGgbl  ftParmUserVECgbl
global ftInputScaleVECgbl ftOutputScaleVECgbl
global ftIScaleValMATgbl  ftOScaleValMATgbl 
global ftIScaleFLGgbl     ftOScaleFLGgbl
%====================================================================

%
% Ceiling for RS parameters
%
if isempty(params), return; end

%
%      Identify the starting point of parameter
%      list for the partitions pertaining to this
%      model.

        i = ftPartInfoMATgbl(4,ftPartIDXgbl);
        i=1;
%---------------------------------------------
% load learned antecedent ftFSrule_ structure
% from the parameter list (params) given by the
% learning scheme such as GA into ftFSainlMATgbl
%---------------------------------------------

	for(idx=1:ftEFMilrnrsCNTgbl)
		ftFSinlMATgbl(ftEFMrinrsCNTgbl(idx),ftEFMcinrsCNTgbl(idx)) = params(i);
		i = i + 1;
	end % for idx loop	
%---------------------------------------------
% load learned consequent ftFSrule_ structure
% from the parameter list (params) given by the
% learning scheme such as GA into ftFSaoutlMATgbl
%---------------------------------------------
	if ftFSfuztypeSCAgbl ~= 2 %NOT TSK
  	  for(idx=1:ftEFMolrnrsCNTgbl)
		if(params(i)>ftGApmaxVECgbl(i))
			params(i)=ftGApmaxVECgbl(i);
		end;
		ftFSoutlMATgbl(ftEFMroutrsCNTgbl(idx),ftEFMcoutrsCNTgbl(idx)) = params(i);
		i = i + 1;
	  end % for idx loop	
	else % if ftFSfuztypeSCAgbl
	  i2=i-1;
	  idx4=0;
	  for idx1=1:ftFSNo_rulesCNTgbl
		if(ftFSrulelearnVECgbl==2)
			for idx2=1:ftFSNo_outCNTgbl
			  i1=i2+1;
			  i2=i1+ftFSNo_inCNTgbl;
			  idx4=idx4+1;
			  isize=i2-i1+1;
			  idx3=(idx1-1)*ftFSNo_outCNTgbl+idx2+1;
			  ftFSoparmMATgbl(idx3,1:isize)=params(i1:i2);
			end
		end
	  end
	end % if ftFSfuztypeSCAgbl
%----------------------------------------------
% load the learned fuzzy membership function
% parameters into the ftEFMparmMATgbl matrix
%----------------------------------------------

%%%%%%%%%%%%%
% FINE TUNING
% 
 	if(ftEFMlrnoptSCAgbl > 4)
	  jj=1;
	  if ftFSfuztypeSCAgbl==2, i=i2+1; end
	  for(ii=i:ftEFMtotlrnCNTgbl)		
		temp1= (params(ii)*ftFSbase_diffSCAgbl)+FTdel_fmfparm(jj);
		if((temp1>=ftFSbase_minSCAgbl)&(temp1<=ftFSbase_maxSCAgbl))
	  		params(ii)=temp1;
		else
			params(ii)=FTdel_fmfparm(jj);
		end; %if
		jj=jj+1;
	  end; % for
	end; %if
%%%%%%%%%%%%%	
	j = 1;
	idx=1;
	while(j<=ftEFMlrnfmfCNTgbl)
		fmfshape=ftioLVtypeVECgbl(ling_idx(idx));
		fmfpcnt = ftFSfmfcntVECgbl(fmfshape);
		if(fmfshape==1) % ftFSfmfSTRgbl is triangular
		  if(idx<=inlrnlvcnt)
			ftFSiparmMATgbl(veciidx(idx),1)=ftmaptou(params(i),3,vecimap(idx));
		  else
			ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),1)=ftmaptou(params(i),4,vecomap(idx-inlrnlvcnt));
		  end; % if
		  adjusts=ftFSbase_maxSCAgbl-params(i);
		  i = i + 1;
		  j = j + 1;
		 for(k=2:3)
		  if(params(i)>adjusts)
			params(i)=adjusts;
		  end; 		
		  if(params(i)<0)
			adjusts=adjusts+params(i);
		  else
		  	adjusts=adjusts-params(i);
		  end; %if
		  if(idx<=inlrnlvcnt)
			ftFSiparmMATgbl(veciidx(idx),k)=ftFSiparmMATgbl(veciidx(idx),k-1)+ abs(ftmaptou(params(i),5,vecimap(idx)));
		  else
			ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),k)=ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),k-1)+abs(ftmaptou(params(i),6,vecomap(idx-inlrnlvcnt)));
		  end; % if
		  i = i + 1;
		  j = j + 1;
		 end; %for k loop
		elseif(fmfshape==2) % ftFSfmfSTRgbl is trapezoidal
		  if(idx<=inlrnlvcnt)
			ftFSiparmMATgbl(veciidx(idx),1)=ftmaptou(params(i),3,vecimap(idx));
		  else
			ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),1)=ftmaptou(params(i),4,vecomap(idx-inlrnlvcnt));
		  end; % if
		  adjusts=ftFSbase_maxSCAgbl-params(i);
		  i = i + 1;
		  j = j + 1;
		 for(k=2:4)		
		  if(params(i)>adjusts)
			params(i)=adjusts;
		  end; 
		  if(params(i)<0)
			adjusts=adjusts+params(i);
		  else
		  	adjusts=adjusts-params(i);
		  end; %if
		  if(idx<=inlrnlvcnt)
			ftFSiparmMATgbl(veciidx(idx),k)=ftFSiparmMATgbl(veciidx(idx),k-1)+abs(ftmaptou(params(i),5,vecimap(idx)));
		  else
			ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),k)=ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),k-1)+abs(ftmaptou(params(i),6,vecomap(idx-inlrnlvcnt)));
		  end; % if
		  i = i + 1;
		  j = j + 1;
		 end; %for k loop

		else % all other ftFSfmfSTRgbl shapes

		  for(k=1:fmfpcnt)
			if((fmfshape==3)&(k==2)) % if FMF type is gaussian
			   passtype_ = 5;
			elseif((fmfshape ==4)&(k==2))  % if FMF type is generalized bell
			   passtype_ = 7;
			elseif((fmfshape ==5)&(k==3))  % if FMF type is exponential
			   passtype_ = 8;
			else    		
			   passtype_ = 3;	
			end; %if
			if(idx<=inlrnlvcnt)
			        ftFSiparmMATgbl(veciidx(idx),k)=ftmaptou(params(i),passtype_,vecimap(idx));
				if((fmfshape==3) & (k==2)) %Gaussian - SD
				   xxx=ftFSiparmMATgbl(veciidx(idx),k);
				   if ftEFMlrnoptSCAgbl > 4
					xxx=max(0.01,abs(xxx/1));
				   else
					xxx=max(0.01,xxx);
				   end
				   ftFSiparmMATgbl(veciidx(idx),k)=xxx;
				end;	%if
			else
				if(passtype_== 3 | passtype_ == 5)
					passtype_ = passtype_ + 1;
				end; %if
			        ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),k)=ftmaptou(params(i),passtype_,vecomap(idx-inlrnlvcnt));
				if((fmfshape==3) & (k==2)) %Gaussian - SD
				   xxx=ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),k);
				   if ftEFMlrnoptSCAgbl > 4
					xxx=max(0.01,abs(xxx/1));
				   else
					xxx=max(0.01,xxx);
				   end
				   ftFSoparmMATgbl(vecoidx(idx-inlrnlvcnt),k)=xxx;
				end;	%if
			end; % if
			i = i + 1;
			j = j + 1;
		  end; % for k loop	
		end; % if

		idx = idx + 1;
	end % for j loop

%
% Input Scaling (if applicable)
%
%
if(ftIScaleFLGgbl)
	ftInputScaleVECgbl(1,1:No_Inputs) = parm(i+1:No_Inputs+i);
	i		    = i+No_Inputs;
end;

%
% Ouput Scaling
%
if(ftOScaleFLGgbl)
	ftOutputScaleVECgbl(1,1:No_Outputs) = parm(i+1:No_Outputs+i);
	i		    = i+No_Outputs;
end;

%
% ADD User Specific parameters here
%
k=0;
if(ftParmUserFLGgbl)
   for(cnt=1:ftPartUserCNTgbl)
    ftParmUserVECgbl(ftUserInfoMATgbl(3,cnt):ftUserInfoMATgbl(4,cnt))=parms(i+1:ftUserInfoMATgbl(5,cnt)+i);
    i=i+ftUserInfoMATgbl(5,cnt);
   end;
end;

%--------------------------------------------------------
% Generate the ftFSbasisMATgbl matrix that generates the membership 
% function curves for all the linguistic variables
%--------------------------------------------------------		                  

ftusecal;
ftfsld;

%  ========================================================================   
%  CAUTION   
%  ========================================================================   
%  EXAMPLE   
   
%  ========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ========================================================================   
%  REFERENCE   
%    
%  Guidelines for Evolutionary Fuzzy Modeling
%  FIG LLC, Tuscaloosa, AL 35486-1477, USA, FIGLLC@AOL.COM
%
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftefmeld.m     
% ftefmeld                           ftefmeld.m                  ftefmeld
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

