%
% ftenhelp.m
%
% =========================================================================  
%                          FlexTools(FS) for Matlab                          
%                             Copyright (c) 1996                               
%                       Flexible Intelligence Group (FIG)       	             
%                   P. O. Box 861477, Tuscaloosa, AL 35486-0013, USA.            
%                                 06/05/1996                                 
% =========================================================================  
%
% FlexTool(ENM) Help options
%
function ftenhelp(help_no)

global Hndle ftHLPlevelSTRgbl ftEN_HelpTopicVECgbl Help_FigHndle ftXYscaleVECgbl

ftHLPlevelSTRgbl=1;
STRING=['hlp' num2str(ftHLPlevelSTRgbl) 'en' num2str(help_no)];

if help_no==1,

   hlp1en1=str2mat(...
            'Neural Modeling involves the application of neural network',...
            'architecture and the associated neural operations and learning methods',... 
            'to develop system models even in extremely uncertain and imprecise',...
            'environments. An integral part of the neural system are the',...
            'associations (weights and connections) which provides IO system mapping.');

   hlp1en1=str2mat(hlp1en1,...
            'We have several options for build the Neural Network:',... 
            '(a) ad-hoc weights/connections (usually a disaster)',...
            '(b) expert a priori knowledge (difficult to get experts to come up with',... 
            '    info.). Useful only for embedding very specific knowledge. ',... 
            '(c) learn Neural Network weights/connections using NN learning',...
            '(d) learn Neural Network weights/connections using EA learning',... 
            '(e)combine a priori knowledge with newly learned weights/connections/etc.');
 
   hlp1en1=str2mat(hlp1en1,...	
            'This document focuses on Evolutionary Neuro Modeling (ENM)',...
            'where we use Evolutionary Algorithms (EA) to evolve the Neural',... 
            'topology, weights, best input, etc..',...
            'The Evolutionary Neuro Modeling (ENM) algorithm given',... 
            'below is straightforward to implement.');

    hlp1en1=str2mat(hlp1en1,...
            '1. Identify the data set or the equations of the system to be controlled.',...
            '2. Identify the inputs and outputs (states and actions).',...
            '3. Select the type of Neural Network to be used',...
            '   Back Prop., RBF, etc. and identify it''s parameters.');

    hlp1en1=str2mat(hlp1en1,...        
            '4. Select a suitable EA learning Scheme.',... 
	    '5. Select the number of input, output, hidden neurons',...
            '6. Select the type of EA, its coding, selection, and',... 
            '   replacement schemes, stopping criteria, and performance',... 
            '   parameters such as population size, mutation rate, and crossover rate.');

    hlp1en1=str2mat(hlp1en1,...            
            '7. Incorporate a priori knowledge. This could be known weights',...
            '   connections, and so on.',...
            '8. Identify appropriate objective function',... 
            '   with fitness computation equations.',...
            '9. Select a suitable sequence set using the Sequencer',... 
            '9. Let EA/GA evolve the neural model parameters',... 
            '   such as weights, topology, best input, etc. off-line.');

    hlp1en1=str2mat(hlp1en1,...
            '10. Identify multiple neuro models, and establish solution',...
            '    hierarchy based on secondary cost functions, if applicable.',...
            '11. Verify performance accuracy, efficiency, and robustness.',...
            '12. Test the system off-line and on-line.');

elseif help_no==2

    hlp1en2=str2mat(...
	    ' Learning Schemes for ENM',...
	    '   (1) Complete (EA learn Weights & Best Input set)',...
            '   (2) Weights  (EA learn All weights)',...
	    '   (3) Connections (EA learn All connections)',...
            '   (4) Partial/Sequencer',...
	    '        Maximum 4 Sequences. Options for learning using:',...
	    '        (a) EA/NN',...
	    '        (b) Specify Duration (EA:# generations, NN: Epochs.',...
	    '        (c) Weights/Connections/Best Input');
    hlp1en2=str2mat(hlp1en2,...
	    '            Note: Do not use weights & Connections together!',...
	    '            Weights, Connections:',...
	    '               None, All, All Fwd, All Rec., Manual',...
	    '               Manual option pops up another window.',...
	    '        Recommended Option: 1. EA, Connections, All;',...
	    '        (Sequence 1,2,3)    2. EA, Weights, All',...
	    '                            3. NN, selected use NN learning',...
	    '   (5) Best Inputs (EA learn best input set, then NN learning)', ...
            '   (6) Fine Tuning Weights (Will be added in next release)');
    hlp1en2=str2mat(hlp1en2,...
	    ' Two choices are provided for ENM RBF learning:',...
	    ' (1) Neuron Level Learning: Learns the standard deviation',...
	    '     For all the neurons (matrix).',...
	    ' (2) Network Level Learning: Here the standard deviation',...
	    '     will be a vector (one for each input type).',...
	    ' EA RBF learning combines EA learning (learning standard deviation',...
	    ' with NN learning (standard RBF learning: identifying mean, minimal',...
	    ' set of neurons, etc.');
elseif help_no==3

    hlp1en3=str2mat(...
	    'Neural System Types:',...
	    '  (a) Feed Forward BackPropogation NN (BP)',...
	    '  (b) Recurrent BackPropogation NN (RBP)',...
            '  (c) Feed Forward Layered BackProp NN',...
            '  (d) Radial Basis Function NN (RBF)');

elseif help_no==4    
    hlp1en4=str2mat(...
            'Artificial Neural Networks (ANNs) or simply neural networks (NN) ',...
            'can be described as either as connectionist models, parallel ',...
            'distributed processing networks, or neuromorphic systems. In ',...
            'general, ANNs consist of many identical nonlinear computational',...
            'elements that are connected in patterns that resemble biological',...
            'neural networks. These simple computational elements, also known ',...
            'as neurons, when connected in complex patterns, have the ability',...
            'to perform memory recall, pattern recognition, and learning.',...
            'Neural Networks have found world-wide acceptance due to their ability',...
            ' to map complex information, and learn from repeated data presentation.');
    
elseif(help_no==5)

   hlp1en5=str2mat(...
            'EA/GA work with a coding of the parameter set, not',... 
            'the parameters themselves, search from a population',...
            'of points, not a single point, and use probabilistic,',...
            'not deterministic transition rules.',... 
            'Learning capabilities, near-optimal solutions,',...
            'robustness, generic code structure, capability to incorporate');

   hlp1en5=str2mat(hlp1en5,...
            'a priori knowledge, adaptability, memory, domain independence,',...
            'hybridization, and advanced operators are some of the',...
            'features of EA/GA that make them ideal candidates',... 
            'for Evolutionary Neuro Modeling. Three types of EA/GA are provided:',...
	    '(1) Regular GA, (2) Micro GA, and (3) Flex-GA (proprietory EA algorithm',...
	    'from Flexible Intelligence Group, LLC.)');

elseif (help_no==6)
    hlp1en6=str2mat(...
	    'An innovative option provided in FlexTool(ENM) is to implement a ',...
            'Sequencer to combine the EA and NN learning paradigms. The sequencer',...
	    'manages the task of alternating between various EA search based learning',...
	    'and NN learning (based on temporal and cost specifications). Its ',...
	    'effectiveness is domain dependent. One useful option is to set up a',...
	    '3 phase sequencer. 1. initially let EA evolve the NN topology ',...
	    '(associations. the fitness being computed based on the output given',...
	    'by the NN) 2. let EA evolve the NN weights (the fitness being computed',...
	    'based on the output given by the NN 3. then allow the NN learning scheme',...
	    'to take over and perform NN learning.');

elseif (help_no==7)
     hlp1en7=str2mat(...
            'It is a known fact that a simple GA converges to a single optimum',...
            'even though multiple peaks of equal quality may exist.  This creates',...                
            'problems in identifying those peaks.  In nature, such a predicament',...
            'is addressed through the formation of stable subpopulations of organisms',...
            'surrounding separate niches by forcing similar individuals to',... 
            'share their resources.  This idea has been introduced in EA/GA literature',...
            'as Niching.  Options for inducing niching includes',...
            'the usage of sharing functions');

    hlp1en7=str2mat(hlp1en7,...
            'Niching helps to maintain subpopulations near global',... 
            'and local optima by introducing a controlled competition',...
            'among different solutions near every global and local optimal region',...
            'Various researchers have used EA/GA with niching to identify multimodal',...
            'solutions. However, when there are several closely lying niches it ',...
            'is difficult to identify them using visual observation. The ENMCN ',...
            'algorithm employs a suitable clustering technique on these niches ',...
            'that evolve among the members of a population to identify the ',...
            'multiple neuro models');

elseif (help_no==8)
    hlp1en8=str2mat(...
	    'FlexTool(ENM) uses niching to evolve multiple neural',...
	    'Models using the ENMCN (Evolutionary Neuro Modeling with',...
	    'Clustered Niches algorithm.');

elseif (help_no==9)
    hlp1en9=str2mat(...
	    'Objective Function / Fitness function is an user provided function',...
            'used as a performance measure to evaluate the performance of the',...
 	    'model. It usually includes a performance index (PI) with penalties',...
            'and rewards for specific system behavior. This is required in ENM',...
            'for evolving the Neural System parameters using EA/GA.',...
            ' ');
    hlp1en9=str2mat(hlp1en9,...
	    'EXAMPLE OBJECTIVE FUNCTION',...
            ' ',...
            'function pit=ftfstu10(PP) %PP has the GA chosen parameters',...
	    ' ',...
	    'ftenmeld(PP); %convert ENM to NN',...
	    ' ',...
	    '% "pit" is the fitness function. Need one every time',...
	    '% FlexTool(ENM) is used');
   hlp1en9=str2mat(hlp1en9,...
	    ' ',...
	    'pit=0; %initialize fitness function',...
	    ' ',...
	    '% Define training data or model definition.',...
	    '% Here we have two inputs (x1,x2); 3 data points each;',...
	    '%  two outputs (y1,y2); the output equations to be mapped',...
	    '%  is given below.',...
	    ' ');
   hlp1en9=str2mat(hlp1en9,...
	    '% Input (3 patterns * 3 input)',...
	    'nn_in= [0.1    0.1    0.1;',...
	    '	     0.1    0.1	   0.9;',...
     	    '        0.1    0.9	   0.1];',...
	    '% Desired Output (4 patterns * 1 output)',...
	    'nn_desired=  [.1;',...
	    '	           .2;',...
            '		   .3];');
   hlp1en9=str2mat(hlp1en9,...
	    '% selection is RBF',...
	    'if(ftNNtypeSCAgbl==14)',...
	    '	ftNNsimFLGgbl   = 0;',...
	    '	ftNNtrainFLGgbl = 1;',...
	    '	nn_out = ftnntrn(nn_in, nn_desired);',...
	    'else',...
	    '	ftNNsimFLGgbl   = 1;',...
	    '	ftNNtrainFLGgbl = 0;',...
	    '	nn_out=ftnntrn(nn_in);',...
	    'end;');
   hlp1en9=str2mat(hlp1en9,...
	    '% Compute total Fitness. Example:',...
	    'PI=norm(nn_out-nn_desired);');
elseif (help_no==10)
    hlp1en10=str2mat(...
	    'FlexTool(ENM) handles multiple objectives');

end; % if


ftpp1_=ftEN_HelpTopicVECgbl((help_no),:);
ftpp2_=eval(STRING);
helpdlg(ftpp2_,ftpp1_)

%
% end ftfshelp.m
%

