%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftfsdcoa                          ftfsdcoa.m                 ftfsdcoa
% =========================================================================  
%                          FlexTools(FS) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
%  =========================================================================  
%  =========================================================================   
%  PURPOSE   
%		Computes the COA (Center of Area or centroid) - defuzzifier
%  .........................................................................   
%  GIVEN   
%       degree - normalized conjunction of rules degree(1:ftFSsteps_)
%       base   - base set (ex. [-1,0.1,1] [start,step,stop] ) base(1:ftFSsteps_)
%       ftFSsteps_  - number of elements (ftFSsteps_)- granularity    
%	dummy1 - dummy variable for generalizing the defuzzification algorithm
%	dummy2 - dummy variable for generalizing the defuzzification algorithm
%  ..........................................................................   
%  OPERATION   
%         Compute the centroid
%  ..........................................................................   
%  RETURNS   
%         y, the centroid
%  ..........................................................................   
%  APPLICATION   
%         Fuzzy Systems, Evolutionary Fuzzy Modeling
%  ..........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%  ==========================================================================   
%   SYNTAX  /  USAGE            
%   y=ftfsdcoa(degree(1:ftFSsteps_), base(1:ftFSsteps_), ftFSsteps_, dummy1 dummy2)
%   y=ftfsdcoa(degree, base, ftFSsteps_, dummy1 dummy2)
%  ==========================================================================  

%  FUNCTION DECLARATION   
   function y=ftfsdcoa(degree, base, ftFSsteps_, dummy1, dummy2)   
   
%  ==========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%   
%	Computes an returns the fuzzy centroid of curve degree
%
%  ==========================================================================   
%  VARIABLE  DICTIONARY   
%       degree - normalized conjunction of rules degree(1:ftFSsteps_)
%       base   - base set ( [0,0.1,1] [start,step,stop] ) base(1:ftFSsteps_)
%       ftFSsteps_  - number of elements (ftFSsteps_)- granularity 
%	dummy1 - dummy variable for generalizing the defuzzification algorithm
%	dummy2 - dummy variable for generalizing the defuzzification algorithm
%  ==========================================================================   
%  ALGORITHM   
%       10      compute sum of product of base values of the consequent
%		(base) and the membership of that value: sumprod
%	20	compute sum of base values of the consequent: sumbase
%	30	compute COA = sumprod/sumbase
%  ==========================================================================   
%  CODE

sumprod = degree(1:ftFSsteps_)*base(1:ftFSsteps_)';
sumbase = sum(degree(1:ftFSsteps_));

if(sumbase)
	y = sumprod/sumbase;
else
	y=0;
end % if

%  ==========================================================================   
%  CAUTION   
%       *       select the type of defuzzifier carefully
%  ==========================================================================   
%  EXAMPLE   
%  ==========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ==========================================================================   
%  REFERENCE   
%    
%  Guidelines for Evolutionary Fuzzy Modeling
%  FIG LLC, Tuscaloosa, AL 35486-1477, USA, FIGLLC@AOL.COM
%
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfsdcoa.m     
% ftfsdcoa                           ftfsdcoa.m                  ftfsdcoa
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
