%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftfsfcrs                          ftfsfcrs.m                 ftfsfcrs
% =========================================================================  
%                          FlexTools(FS) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)                            
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
% =========================================================================  
% =========================================================================   
%  PURPOSE   
%         Run the fuzzy system engine and generate output  
%                 GENERIC REGULAR FUZZY CONTROLLER
%  .........................................................................   
%  GIVEN   
%                 Input values for the Fuzzy System
%  .........................................................................   
%  OPERATION   
%
%                 Perform fuzzy computations and generate output
%  .........................................................................   
%  RETURNS   
%         Output of the fuzzy system
%  .........................................................................   
%  APPLICATION   
%         Fuzzy Systems, Evolutionary Fuzzy Modeling
%  .........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%  ========================================================================   
%   SYNTAX  /  USAGE  
%   ftFSout_ = ftfsfcrs(x)
%  ========================================================================  

%  FUNCTION DECLARATION   
   function ftFSout_ = ftfsfcrs(x)
   
%  ========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%
%   
%  ========================================================================   
%  VARIABLE DICTIONARY   
%
%  ========================================================================   
%  ALGORITHM   
%           GENERIC REGULAR FUZZY CONTROLLER
%
%               05      Map input from universe of discourse to base set
%               10      Compute the membership of the input values in the antecedent fuzzy 
%                       subsets (using appropriate fuzzy conjunction or disjunction)
%               20      Apply appropriate fuzzy implication to generate the fuzzy output
%                       subset for each rule
%               30      Apply suitable fuzzy aggregation function to ftFSaggregateSTRgbl the ftFSaction_
%                       subsets of all rules.
%               40      Normalize the resultant of aggregation subset for the rulebase.
%               50      Apply suitable defuzzification function to generate a crisp output.
%               60      Repeat 20-50 ftFSNo_outCNTgbl times
%  ========================================================================   
%  CODE
%
% GLOBAL
global ftFSbase_minSCAgbl ftFSbase_maxSCAgbl ftFSNo_stepsCNTgbl ftFSncountCNTgbl ftFSbasisMATgbl xsave 
global ftFSNo_rulesCNTgbl ftFSNo_critCNTgbl ftFSNo_lingCNTgbl No_pnam ftFSmaxsVECgbl ftFSminsVECgbl ftFSNo_inCNTgbl ftFSNo_outCNTgbl
global ftFSaincMATgbl ftFSainlMATgbl ftFSainrMATgbl ftFSaoutcMATgbl ftFSaoutlMATgbl ftFSaoutrMATgbl ftFSbase_setVECgbl ftFSparmMATgbl
global ftFSfmfSTRgbl No_fmfp ftFSaggrparmMATgbl ftFSdefparmMATgbl l_rmap l_cmap
global ftFSfmfilenMATgbl ftFSiLVtypeVECgbl ftFSfmfcntVECgbl ftioLVtypeVECgbl
global ftFSimpnameMATgbl ftFSaggrfilenMATgbl ftFSdefuznameMATgbl
global ftFSimptypeSCAgbl ftFSaggrtypeSCAgbl ftFSdeftypeSCAgbl
global ftFSsum_lingVECgbl ftFSNo_ilingCNTgbl

%====================================================================

%
% Map the input values from the universe of discourse to the base set
%
x = ftmaptob(x,1);
for r = 1:ftFSNo_rulesCNTgbl
  %------------------------------------------------------------------
  %    Have to generate the matrix "basis"
  %    Normalize this curve
  %    Find the membership values for each criteria in the rules
  %------------------------------------------------------------------
  ftFSstate_ = 1;
  while(ftFSstate_<=ftFSNo_inCNTgbl)
    if(ftFSaincMATgbl(r,ftFSstate_) ~= 0)

    %------------------------------------------------------------
    % APPLY
    % FUZZY MEMBERSHIP FUNCTION: users choice - in the string ftFSfmfSTRgbl
    %------------------------------------------------------------
        ftFSloc_ = ftFSainlMATgbl(r,ftFSstate_)+ftFSsum_lingVECgbl(ftFSaincMATgbl(r,ftFSstate_));
        cnt = ftFSfmfcntVECgbl(ftioLVtypeVECgbl(ftFSloc_));
        ftFSfmfparmMATgbl(1:cnt)= ftFSparmMATgbl(ftFSloc_,1:cnt);
        memb_val(r,ftFSstate_) = feval(ftFSfmfilenMATgbl(ftioLVtypeVECgbl(ftFSloc_),:), x(ftFSaincMATgbl(r,ftFSstate_)), ftFSfmfparmMATgbl);
    %----------------------
    %  normalize this value
    %----------------------
    if ftFSmaxsVECgbl(ftFSloc_)==ftFSminsVECgbl(ftFSloc_)
     memb_val(r,ftFSstate_)=0;
    else
     memb_val(r,ftFSstate_) = (memb_val(r,ftFSstate_)-ftFSminsVECgbl(ftFSloc_))/(ftFSmaxsVECgbl(ftFSloc_)-ftFSminsVECgbl(ftFSloc_));
    end
   end; % if
   ftFSstate_ = ftFSstate_ + 1;
  end; % while loop
end % for r loop

%=========================================================================
%-------------------------------------------------------------------------
%   Apply the rule (combination of criteria, linguistic variable , AND|OR)
%-------------------------------------------------------------------------
for r = 1:ftFSNo_rulesCNTgbl
   ftFSstate_ = 1;
   while(ftFSstate_<=(ftFSNo_inCNTgbl-1))
    if(ftFSaincMATgbl(r,ftFSstate_) ~= 0)
     if(ftFSainrMATgbl(r,ftFSstate_) == 1)
       memb_val(r,1) = min(memb_val(r,1),memb_val(r,(ftFSstate_+1)));
     elseif(ftFSainrMATgbl(r,ftFSstate_) == 2)
       memb_val(r,1) = max(memb_val(r,1),memb_val(r,(ftFSstate_+1)));
     end % if
    end % if
    ftFSstate_ = ftFSstate_ + 1;
   end % while
end % for r loop
%=========================================================================
%------------------------
% One value for each ftFSrule_
%------------------------
ftFSrule_ = memb_val(:,1);

%=========================================================================
%-------------------------
% For each ftFSaction_ criteria
%--------------------------
for act=1:ftFSNo_outCNTgbl
   %----------------------------------------------
   % Set up ftFSrbasisMATgbl according to the type of ftFSaction_
   %----------------------------------------------
   ftFSrbasisMATgbl(1:ftFSNo_rulesCNTgbl,:) = ftFSbasisMATgbl(ftFSaoutlMATgbl(1:ftFSNo_rulesCNTgbl,act) + ftFSNo_ilingCNTgbl,:);
   %figure
   %plot([-1:0.01:1],ftFSrbasisMATgbl)
   %pause
   %--------------------------------------------------------------------------
   % FUZZY IMPLICATION: choosen implication function - in the string ftFSimplicateSTRgbl
   %--------------------------------------------------------------------------
   imply = feval(ftFSimpnameMATgbl(ftFSimptypeSCAgbl,:),ftFSrule_, ftFSrbasisMATgbl, ftFSNo_rulesCNTgbl, ftFSNo_stepsCNTgbl);
   %figure
   %plot([-1:0.01:1],imply)
   %pause
   %---------------------------------------------------------
   %  FUZZY AGGREGATION: use the choosen aggregation function
   %---------------------------------------------------------
   if(ftFSaggrtypeSCAgbl==15)  
                aggr = ftfsgenm(imply, ftFSNo_rulesCNTgbl, ftFSaggrparmMATgbl(ftFSaggrtypeSCAgbl,:))
   else 
                if(ftFSNo_rulesCNTgbl==1)
                        aggr = imply;
                else
                        aggr=imply(1,:);

                        for r=2:ftFSNo_rulesCNTgbl
                                aggr(1:ftFSNo_stepsCNTgbl)=feval(ftFSaggrfilenMATgbl(ftFSaggrtypeSCAgbl,:),aggr(1:ftFSNo_stepsCNTgbl),imply(r,1:ftFSNo_stepsCNTgbl),ftFSaggrparmMATgbl(ftFSaggrtypeSCAgbl,:));
                        end % for r loop
                end % if
   end % if
   
   %--------------------------------------
   %  Normalize the Aggregated Fuzzy Curve
   %-------------------------------------- 
   norms  = ftfsnorm(aggr);
   %figure
   %plot([-1:0.01:1],norms)
   %pause
   %---------------------------------------------------------------
   %  DEFUZZIFIER. (defuzzify the Normalized Fuzzy Aggregate Curve)
   %                            use the choosen defuzzifier
   %---------------------------------------------------------------
   ftFSout_(act)  = feval(ftFSdefuznameMATgbl(ftFSdeftypeSCAgbl,:),norms, ftFSbase_setVECgbl, ftFSNo_stepsCNTgbl, ftFSdefparmMATgbl(ftFSdeftypeSCAgbl,1), ftFSdefparmMATgbl(ftFSdeftypeSCAgbl,2));

end % for act loop

%
% Map the output values from the base set to the universe of discourse
%
ftFSout_ = ftmaptou(ftFSout_,2);

%
%  ========================================================================   
%  CAUTION   
%  ========================================================================   
%  EXAMPLE   
   
%  ========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ========================================================================   
%  REFERENCE   
%    
%  Guidelines for Evolutionary Fuzzy Modeling
%  FIG LLC, Tuscaloosa, AL 35486-1477, USA, FIGLLC@AOL.COM
%
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfsfcrs.m     
% ftfsfcrs                           ftfsfcrs.m                  ftfsfcrs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


