%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftfsld                          ftfsld.m                 ftfsld
% =========================================================================  
%                          FlexTools(FS) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)                 
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
% =========================================================================  
% =========================================================================   
%  PURPOSE   
%         Load the fuzzy ftFSrule_ (antecedents and consequents linguistic
%         variable matrix) and membership function parameter matrix.
%  .........................................................................   
%  GIVEN  
%                 fuzzy parameters 
%  .........................................................................   
%  OPERATION   
%
%                 Load parameters in appropriate matrices
%  .........................................................................   
%  RETURNS   
%         Nothing. Global matrices get loaded.
%  .........................................................................   
%  APPLICATION   
%         Fuzzy Systems, Evolutionary Fuzzy Modeling
%  .........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%  ========================================================================   
%   SYNTAX  /  USAGE  
%   ftfsld(params)
%  ========================================================================  

%  DECLARATION   

function ftfsld(params)
   
%  ========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%
%   
%  ========================================================================   
%  VARIABLE DICTIONARY   
%
%  ========================================================================   
%  ALGORITHM   
%   
%10     Compute the membership of the input values in the antecedent fuzzy 
%       subsets (using appropriate fuzzy conjunction or disjunction).
%  ========================================================================   
%  CODE
%
% GLOBAL
global ftFSncountCNTgbl ftFSbasisMATgbl xsave 
global ftFSNo_rulesCNTgbl ftFSaNo_critCNTgbl ftFSNo_lingCNTgbl No_pnam  ftFSNo_stepsCNTgbl ftFSmaxsVECgbl ftFSminsVECgbl No_action ftFSaNo_lingCNTgbl
global ftFSaincMATgbl ftFSainlMATgbl ftFSainrMATgbl ftFSaoutcMATgbl ftFSaoutlMATgbl ftFSaoutrMATgbl ftFSbase_setVECgbl ftFSparmMATgbl
global ftFSfmfSTRgbl No_fmfparm ftFSaggrparmMATgbl ftFSdefparmMATgbl  ftFSfmfparmMATgbl 
global ftFSsum_lingVECgbl lengthmatin lengthmatout lengthmatfmf
global in_r_idx in_in_idx out_r_idx out_out_idx
global No_fmfp crit_idx ling_idx 
global ftioLVtypeVECgbl ftFSfmfcntVECgbl ftFSfmfilenMATgbl ftFSfuncfilenMATgbl
global ftFSmaxsVECgbl ftFSminsVECgbl ftFSfuztypeSCAgbl ftFSNo_outCNTgbl

%====================================================================
%--------------------------------------------------------
% Generate the ftFSbasisMATgbl matrix that generates the membership 
% function curves for all the linguistic variables
%--------------------------------------------------------                                 
 j = 1;
 if ftFSfuztypeSCAgbl ==2, 
   iend=ftFSaNo_critCNTgbl-ftFSNo_outCNTgbl;
 else
   iend=ftFSaNo_critCNTgbl;
 end
 for(i=1:iend)               
  for(l=1:ftFSaNo_lingCNTgbl(i))                 
    cnt               = ftFSfmfcntVECgbl(ftioLVtypeVECgbl(j));
    fmfparms(1:cnt)    = ftFSparmMATgbl(j,1:cnt);
    ftFSdummy_ = feval(ftFSfmfilenMATgbl(ftioLVtypeVECgbl(j),:),ftFSbase_setVECgbl(1:ftFSNo_stepsCNTgbl)',fmfparms);
    ftFSbasisMATgbl(j,1:ftFSNo_stepsCNTgbl) = ftFSdummy_';
    ftFSmaxsVECgbl(j) = max(ftFSbasisMATgbl(j,:));
    ftFSminsVECgbl(j) = min(ftFSbasisMATgbl(j,:));                         
    den = ftFSmaxsVECgbl(j)-ftFSminsVECgbl(j);
    if(den==0)
        ftFSbasisMATgbl(j,:)=zeros(1,ftFSNo_stepsCNTgbl);
    else        
        ftFSbasisMATgbl(j,:) = (ftFSbasisMATgbl(j,:) - ftFSminsVECgbl(j))/den;
    end % if    
    j = j + 1;
  end % for l loop
 end % for i loop 

%  ========================================================================   
%  CAUTION   
%  ========================================================================   
%  EXAMPLE   
   
%  ========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ========================================================================   
%  REFERENCE   
%    
%  Guidelines for Evolutionary Fuzzy Modeling
%  FIG LLC, Tuscaloosa, AL 35486-1477, USA, FIGLLC@AOL.COM
%
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfsld.m     
% ftfsld                           ftfsld.m                  ftfsld
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


