%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftfsnorm                          ftfsnorm.m                 ftfsnorm
% =========================================================================  
%                          FlexTools(FS) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG) 
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
%  =========================================================================  
%  =========================================================================   
%  PURPOSE   
%       normalizing function
%  .........................................................................   
%  GIVEN   
%       a - vector - unnormalized vector
%  ..........................................................................   
%  OPERATION   
%         normalize the vector
%  ..........................................................................   
%  RETURNS   
%         y, the normalized vector
%  ..........................................................................   
%  APPLICATION   
%         Fuzzy Systems, min., Evolutionary Fuzzy Modeling
%  ..........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%  ==========================================================================   
%   SYNTAX  /  USAGE  
%   y=ftfsnorm(a(1:ftFSsteps_))
%   y=ftfsnorm(a)
%  ==========================================================================  

%  FUNCTION DECLARATION   
   function y=ftfsnorm(a)   
   
%  ==========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%  
%   normalize the vector 
%  ==========================================================================   
%  VARIABLE  DICTIONARY   
%       a - vector - unnormalized conjunction of rules over [0,0.1,1]
%	ftFSdenom_	- denominator
%  ==========================================================================   
%  ALGORITHM   
%       10      find the maximum value of the vector
%	20	find the minimum value
%	30	normalize the vector
%  ==========================================================================   
%  CODE
%

maxval = max(a);
minval = min(a);
ftFSdenom_  = maxval - minval;
if( (isnan(ftFSdenom_)==0)&(isinf(ftFSdenom_)==0)&(ftFSdenom_~=0) )
	y = (a - minval)/ftFSdenom_;
else
	y = a;
end % if
%  ==========================================================================   
%  CAUTION   
% 
%  ==========================================================================   
%  EXAMPLE   
%  ==========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ==========================================================================   
%  REFERENCE   
%    
%  Guidelines for Evolutionary Fuzzy Modeling
%  FIG LLC, Tuscaloosa, AL 35486-1477, USA, FIGLLC@AOL.COM
%
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfsnorm.m     
% ftfsnorm                           ftfsnorm.m                  ftfsnorm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

