%
% ftfssim.m
%
% =========================================================================  
%                          FlexTools(FS) for Matlab                          
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)       	             
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 12/05/1996                                  
% =========================================================================  
% Simulates Fuzzy System for embedding FS in user software
% Usage: [fs_out, ftFSaltidxVECgbl, ftFSactidxVECgbl, ftFSscoresMATgbl]=ftfssim(ftFSfile,fs_in)
%
%
% Given:  
%	  fs_in		        :   Input values to the fuzzy system
%         ftFSfile		:   Name of the file where the FS information is saved
% Uses:
%	  ftFSfuztypeSCAgbl   	:   (Global) index to the type of fuzzy system 
%				    Assumes that the choice is not the fuzzy decision maker.
%	  ftFSfuncfilenMATgbl   :   name of fuzzy systems ( ftFSfuncfilenMATgbl(1:No_opts,:) ) 
%
% Action: select ftFSfuncfilenMATgbl(ftFSfuztypeSCAgbl,:)
%	  run the appropriate fuzzy system.
%	  
% Return: results 
%         fs_out = output of the FS 

function [fs_out, ftFSaltidxVECgbl, ftFSactidxVECgbl, ftFSscoresMATgbl]=ftfssim(ftFSfile,fs_in)

global ftFSfuztypeSCAgbl ftFSinfnameSTRgbl ftscr_cntSCAgbl
global ftFigPos_medVECgbl ftFSbprmSCAgbl
global ftFSmax_rulesSCAgbl ftFSmax_LVSCAgbl ftFSmax_inSCAgbl ftFSmax_outSCAgbl ftFSmaxsVECgbl ftFSminsVECgbl
global ftFSbase_minSCAgbl ftFSbase_maxSCAgbl ftFSbase_diffSCAgbl ftFSbase_resSCAgbl ftFSbase_setVECgbl ftFSNo_stepsCNTgbl 
global ftFSrbasisMATgbl ftFSbasisMATgbl ftFS_inMATgbl ftFSqual_critMATgbl
global ftFSNo_rulesCNTgbl ftFSNo_critCNTgbl ftFSNo_ilingCNTgbl ftFSNo_olingCNTgbl ftFSaNo_critCNTgbl ftFSNo_altCNTgbl
global ftFSilingVECgbl ftFSimaxVECgbl ftFSiminVECgbl ftFSiunitVECgbl ftFSinameVECgbl ftFSNo_inCNTgbl
global ftFSiLVlearnVECgbl ftFSiLVtypeVECgbl ftFSiLVnameVECgbl 
global ftFSolingVECgbl ftFSomaxVECgbl ftFSominVECgbl ftFSounitVECgbl ftFSonameVECgbl ftFSNo_outCNTgbl
global ftFSoLVlearnVECgbl ftFSoLVtypeVECgbl ftFSoLVnameVECgbl ftFSrulelearnVECgbl ftioLVtypeVECgbl
global ftFSunivMATgbl ftFSlnameMATgbl ftFSincMATgbl ftFSinlMATgbl ftFSinrMATgbl ftFSoutcMATgbl ftFSoutlMATgbl 
global ftFSoutrMATgbl ftFSiparmMATgbl ftFSoparmMATgbl ftFSilrnMATgbl ftFSolrnMATgbl mat_parm
global ftFSdisp1FLGgbl ftFSFLGgbl ftFS2FLGgbl ftflag_edtGUI ftparm_edtGUI
global ftSlide_aGUI ftSlide_bGUI ftSlide_cGUI ftSlide_dGUI ftFt_PlotGUI ftxmingbl ftFSxmaxgbl ftFSdelxgbl
global ftFSvarCNTgbl ftFSlingCNTgbl ftFSbprmSCAgbl
global ftview_countGUI ftview_stringGUI ftFSfig_fsviewGUI
global ftcolor_bc1VECgbl ftcolor_bc2VECgbl ftcolor_fc1VECgbl
global ftFSfmfilenMATgbl ftFSfmfcntVECgbl ftFSimpnameMATgbl ftFSaggrfilenMATgbl ftFSdefuznameMATgbl
global ftFSaggrparmnMATgbl ftFSdefpfilnMATgbl ftFStskbMATgbl
global ftFSfmftypenMATgbl ftFSimptypenMATgbl ftFSaggrtypenMATgbl ftFSdeftypenMATgbl
global fuztitle ftFSfuncfilenMATgbl fig_ftfs
global ftFSdfmfparmMATgbl ftFSdaggrparmMATgbl ftFSddefparmMATgbl ftFSfmfparmMATgbl ftFSaggrparmMATgbl ftFSdefparmMATgbl 
global ftFSaggrpcntVECgbl ftFSdefpcntVECgbl
global ftFSfuztypeSCAgbl ftFSfuzltypeSCAgbl ftFSaggrtypeSCAgbl ftFSdeftypeSCAgbl ftFSimptypeSCAgbl 
global ftFSfmfSTRgbl ftFSimplicateSTRgbl ftFSaggregateSTRgbl ftFSdefuzSTRgbl ftscr_cntSCAgbl fig_fs4
global ftFSyinitnameSTRgbl ftFSyobjname1STRgbl ftFSyobjname2STRgbl ftFSyouriterSTRgbl
global ftFSaincMATgbl ftFSainrMATgbl ftFSainlMATgbl ftFSaoutcMATgbl ftFSaoutrMATgbl ftFSaoutlMATgbl ftFSparmMATgbl
global ftFSsum_lingVECgbl ftFSfig_fsbaseGUI fig_fs22 ftFSN_totalCNTgbl ftFSpfGUI
global ftFigPos_bigVECgbl ftFigPos_medVECgbl ftFigPos_smallVECgbl ftXscaleVECgbl ftYscaleVECgbl ftXYscaleVECgbl
global ftFigPos_VsmallVECgbl
global ftFSallaltMATgbl ftFSactidxVECgbl ftFSaltidxVECgbl ftFSscoresMATgbl
global ftEFMlrnoptSCAgbl
global ftFSaNo_lingCNTgbl ftFSaLVidxVECgbl ftFSaoLVidxVECgbl ftFSmax_LVSCAgbl
global ftFSaNo_critCNTgbl ftFSNo_inCNTgbl ftFSNo_outCNTgbl 
global ftFSilingVECgbl ftFSolingVECgbl 
global ftFSidiffVECgbl ftFSodiffVECgbl ftFSiminVECgbl ftFSominVECgbl ftFSimaxVECgbl ftFSomaxVECgbl
global ftFSiLVtypeVECgbl ftFSoLVtypeVECgbl ftFSiLVlearnVECgbl ftFSoLVlearnVECgbl
global ftFSbase_maxSCAgbl ftFSbase_minSCAgbl ftFSbase_diffSCAgbl
global ftFSNo_ilingCNTgbl ftFSNo_olingCNTgbl ftFSatot_lingVECgbl 
global ftFSincMATgbl ftFSinrMATgbl ftFSinlMATgbl
global ftFSoutcMATgbl ftFSoutrMATgbl ftFSoutlMATgbl
global ftFSaincMATgbl ftFSainrMATgbl ftFSainlMATgbl
global ftFSaoutcMATgbl ftFSaoutrMATgbl ftFSaoutlMATgbl
global ftFSiparmMATgbl ftFSoparmMATgbl ftFSparmMATgbl
global ftFSNo_rulesCNTgbl ftioLVtypeVECgbl ftioLVlearnVECgbl
global ftFSmax_inSCAgbl ftFSmax_outSCAgbl ftFSmax_LVSCAgbl
global ftFSsum_lingVECgbl

global ftFSoldfile

if sum(ftFSoldfile)~=sum(ftFSfile)
   load ft3fs_def
   eval(['load ' ftFSfile]);
%   disp(ftFSfile);
end

ftFSoldfile=ftFSfile;

ftFS_inMATgbl=fs_in;

if(ftusecal)

ftfsld(1);

[ftFSNo_altCNTgbl, dummy_]=size(fs_in);

%
% Call different method for decision maker/evaluator
%

if((ftFSfuztypeSCAgbl==3)|(ftFSfuztypeSCAgbl==4))
	[ftFSallaltMATgbl,ftFSaltidxVECgbl, ftFSactidxVECgbl, ftFSscoresMATgbl] = ftfsfdms(fs_in);
	fs_out=ftFSallaltMATgbl;

%	    Note! the ftFSscoresMATgbl are in the ascending order!, best last!
%	    ftFSaltidxVECgbl - index of alternative (worst to best) - row
%	    ftFSactidxVECgbl - index of ftFSaction_ (for each action!) - column
% 	    second or only part of the user objective function, before
% 	    using the fuzzy system. default is ftefobj9.m which
%	    just returns the input.
%
else
    for (cnt_=1:ftFSNo_altCNTgbl)
        ftFS_inMATgbl = fs_in(cnt_,:);
	fs_out(cnt_,:) = feval(ftFSfuncfilenMATgbl(ftFSfuztypeSCAgbl,:),fs_in(cnt_,:));
    end; % for
end; %if

else
	ftdispwn('Exception Detected!','Attention!! Missing FS parameters!');
end; %if

%
% end ftfssim.m
%


