%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftfstrap                         ftfstrap.m                       ftfstrap
% =========================================================================  
%                          FlexTools(FS) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
%  =========================================================================  
%  =========================================================================   
%  PURPOSE   
%          Trapezoidal Membership function. To obtain membership values   
%  .........................................................................   
%  GIVEN   
%         x	: input value (vector)
%         a	: left (bottom)  limit	(controls width)  parm(1)
%         b 	: left (top)     limit 	(controls height) parm(2)
%	  c	: right (top)    limit  (controls height) parm(3)
%	  d	: right (bottom) limit	(controls width)  parm(4)
%  ..........................................................................   
%  OPERATION   
%         Compute the membership value for x in the MF
%  ..........................................................................   
%  RETURNS   
%         y, the membership value
%  ..........................................................................   
%  APPLICATION   
%         Fuzzy Systems, one shape for MF, Evolutionary Fuzzy Modeling
%  ..........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%       zeros, for, norm
%  ==========================================================================   
%   SYNTAX  /  USAGE  
%   y=ftfstrap(x,parm(1:No_fmfparm)
%   y=ftfstrap(x,parm)   
%  ==========================================================================  

%  FUNCTION DECLARATION   
   function y=ftfstrap(x, parm)   
   
%  ==========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%   
%  The Trapezoidal Fuzzy membership function which computes the membership
%  value for a given input value x.
%  ==========================================================================   
%  VARIABLE  DICTIONARY   
%         x	: input value
%         a	: left (bottom)  limit	(controls width)  parm(1)
%         b 	: left (top)     limit 	(controls height) parm(2)
%	  c	: right (top)    limit  (controls height) parm(3)
%	  d	: right (bottom) limit	(controls width)  parm(4)
%  ==========================================================================   
%  ALGORITHM   
%       10      compute the membership value of x in the trapeziodal
%		membership function specified by a, b, c, and d.
%  ==========================================================================   
%  CODE

nn=size(x,1);
dp1=parm(2)-parm(1);
dp2=parm(3)-parm(4);
for i=1:nn
  if((parm(1)<=x(i))&(x(i)<parm(2)))
	if(parm(2)~=parm(1))
  	  y(i,1)=(x(i)-parm(1))/dp1;
	else
          y(i,1)=1.0;
        end % if
  elseif((parm(2)<=x(i))&(x(i)<=parm(3)))
	y(i,1)=1.0;
  elseif((parm(3)<x(i))&(x(i)<=parm(4)))
	if(parm(3)~=parm(4))
          y(i,1)=(x(i)-parm(4))/dp2;
	else
          y(i,1)=1.0;
        end % if
  else
	y(i,1)=0;
 end % if
end
%  ==========================================================================   
%  CAUTION   
%       *       select the type of membership function carefully
%  ==========================================================================   
%  EXAMPLE   
%  ==========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ==========================================================================   
%  REFERENCE   
%    
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using Genetic Algorithms and its application   
%  AIAA Computing in Aerospace (CIA10) Conference, San Antonio, TX, USA , March 1995.   
%  
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfstrap.m     
% ftfstrap                          ftfstrap.m                  ftfstrap.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


