%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftfstria                         ftfstria.m                  ftfstria
% =========================================================================  
%                          FlexTools(FS) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
%  =========================================================================  
%  =========================================================================   
%  PURPOSE   
%          Triangular Membership function. To obtain membership values   
%  .........................................................................   
%  GIVEN   
%         x	: input value
%         a	: left (lower) value of the triangular MF(controls width)parm(1)
%         b     : centre value of the triangular MF (controls height)parm(2)
%	  c	: right (max) value of the triangular MF (cntrols width) parm(3)
%  ..........................................................................   
%  OPERATION   
%         Compute the membership value for x in the MF
%  ..........................................................................   
%  RETURNS   
%         y, the membership value
%  ..........................................................................   
%  APPLICATION   
%         Fuzzy Systems, one shape for MF, Evolutionary Fuzzy Modeling
%  ..........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%       zeros, for, norm
%  ==========================================================================   
%   SYNTAX  /  USAGE  
%   y=ftfstria(x, parm(1:No_fmfparm)
%   y=ftfstria(x,parm)   
%  ==========================================================================  

%  FUNCTION DECLARATION   
   function yy=ftfstria(xx, parm)   
   
%  ==========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%   
%  The Triangular Fuzzy membership function which computes the membership
%  value for a given input value x.
%  ==========================================================================   
%  VARIABLE  DICTIONARY   
%         x	: input value
%         a	: left (lower) value of the triangular MF   (controls width)
%         b     : centre value of the triangular MF         (controls height)
%	  c	: right (max) value of the triangular MF    (cntrols width)
%  ==========================================================================   
%  ALGORITHM   
%       10      compute the membership value of x in the triangular
%		membership function specified by a, b, and c.
%  ==========================================================================   
%  CODE   

nn=size(xx,1);
for i=1:nn
 x=xx(i);
 if(x==parm(2))
	y=1;
 elseif((parm(1)<=x)&(x<parm(2)))
	if(parm(2)~=parm(1))
		y=((x-parm(1))/(parm(2)-parm(1)));
	else
		y=1.0;
	end % if
 elseif((parm(2)<x)&(x<=parm(3)))
	if(parm(2)~=parm(3))
		y=((x-parm(3))/(parm(2)-parm(3)));
	else
		y=1.0;
	end % if
 else
	y=0;
 end % if
 yy(i,1)=y;
end
%  ==========================================================================   
%  CAUTION   
%       *       select the type of membership function carefully
%  ==========================================================================   
%  EXAMPLE   
%  ==========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ==========================================================================   
%  REFERENCE   
%    
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using Genetic Algorithms and its application   
%  AIAA Computing in Aerospace (CIA10) Conference, San Antonio, TX, USA , March 1995.   
%  
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfstria.m     
% ftfstria                          ftfstria.m                  ftfstria.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


