%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% FTtfsskf                            FTfstskf.m                 FTfstskf
% =========================================================================  
%                          FlexTools(FS) for Matlab                          
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
%  =========================================================================  
%  =========================================================================   
%  PURPOSE   
%          Sugeno (Takagi Sugeno Kang - TSK) Model    
%          To obtain a crips output from the TSK fuzzy controller   
%  .........................................................................   
%  GIVEN   
%         ftFSNo_rulesCNTgbl : Number of Rules (n)
%         ftFSNo_inputs_: Number of states / inputs / antecedents / u (r)   
%         b        : Constants of the linear/non-linear functions   
%                    b(1:ftFSNo_rulesCNTgbl,1:ftFSNo_inputs_+1)     
%         ftFSDOF_      : Degree Of Firing matrix. ftFSDOF_(1:ftFSNo_rulesCNTgbl)          
%         u        : The input / antecedent values. u(1:ftFSNo_inputs_) 
%  ..........................................................................   
%  OPERATION   
%         Computes the ftFSaction_ of TSK model   
%  ..........................................................................   
%  RETURNS   
%         y, the ftFSaction_ / output of the TSK model
%  ..........................................................................   
%  APPLICATION   
%         Fuzzy Controllers, in TSK type
%  ..........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTrunga.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%       zeros, for, norm
%  ==========================================================================   
%   SYNTAX  /  USAGE  
%   y=ftfstskf(ftFSNo_rulesCNTgbl, ftFSNo_inputs_, ftFSDOF_(1:ftFSNo_rulesCNTgbl), b(1:ftFSNo_rulesCNTgbl,1:ftFSNo_inputs_+1), u(1:ftFSNo_inputs_))   
%   y=ftfstskf(ftFSNo_rulesCNTgbl, ftFSNo_inputs_, ftFSDOF_, b, u)   
%  ==========================================================================  

%  FUNCTION DECLARATION   
   function y=ftfstskf(ftFSNo_rulesCNTgbl, ftFSNo_inputs_, ftFSDOF_, b, u)   
   
%  ==========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%   
%  To compute the crisp y value using TSK method
%  from a set of fuzzy rules, and inputs.
%  ==========================================================================   
%  VARIABLE  DICTIONARY   
%     
%         ftFSNo_rulesCNTgbl : Number of Rules (n)
%         ftFSNo_inputs_: Number of states / inputs / antecedents / u (r)   
%         b        : Constants of the linear/non-linear functions   
%                    b(1:ftFSNo_rulesCNTgbl,1:ftFSNo_inputs_+1)     
%         ftFSDOF_      : Degree Of Firing matrix. ftFSDOF_(1:ftFSNo_rulesCNTgbl)          
%         u        : The input / antecedent values. u(1:ftFSNo_inputs_) 
%  LOCAL   
%       k, i, j    : k parameters, ith and jth individuals   
%  ==========================================================================   
%  ALGORITHM   
%       10      compute sum of product of (degree of firing, constant b, 
%		input value of each antecedent/input) for all rules
%       20      compute sum of degree of firing values for all rules
%       30      compute y as the ratio of values obtained in 20 and 30
%	40	return y 
%  ==========================================================================   
%  CODE   
   
ftFSnumer_=0;
ftFSdenom_=sum(ftFSDOF_(1:ftFSNo_rulesCNTgbl));
if(ftFSdenom_==0), ftFSdenom_=0.00001; end 
[n,m]=size(u);
if m == ftFSNo_inputs_
   u=u';
end
temp1 = b(:,1);
temp1 = ( (b(1:ftFSNo_rulesCNTgbl,2:ftFSNo_inputs_+1)*u(1:ftFSNo_inputs_)) + temp1).*ftFSDOF_;

ftFSnumer_=sum(temp1);
y=ftFSnumer_/ftFSdenom_;

%  ==========================================================================   
%  CAUTION   
%       *       select the type of controller carefully
%  ==========================================================================   
%  EXAMPLE   
%  ==========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ==========================================================================   
%  REFERENCE   
%    
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using Genetic Algorithms and its application   
%  AIAA Computing in Aerospace (CIA10) Conference, San Antonio, TX, USA , March 1995.   
%  
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfsskf.m     
% FTfstskf                             FTfstskf.m                 FTfstskf
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
