% =================================================================
% ftnnsim.m
% =================================================================
%                        FlexTool(NN) for Matlab                          
%                           Copyright (c) 1996                               
%                   Flexible Intelligence Group (FIG)       	             
%            P. O. Box 861477, Tuscaloosa, AL 35486-0013, USA.            
%                              06/05/1996                                  
% =================================================================
% Simulates Neural System for embedding NN in user software
% Usage: nn_out=ftnnsim(ftNNfile,nn_in)
% where:
%      nn_out=output of the Neural network
%		 Size=(# of patterns x # of outputs)
%      nn_in=input to the Neural network
%		 Size=(# of patterns x # of inputs)
%      ftNNfile=Name of the file where the NN information is saved

function nn_out_mat=ftnnsim(ftNNfile,nn_in_mat)

global ftNNasetFLGgbl ftNNfnflagSCAgbl ftnndef_flag 
global ftNNlrSCAgbl ftNNmrSCAgbl ftNNtitleVECgbl
global ftNNn_epSCAgbl ftNNe_thSCAgbl ftNNv_pcntSCAgbl 
global ftNNbase_resSCAgbl ftNNbase_minSCAgbl ftNNbase_maxSCAgbl 
global ftNNtypeSCAgbl ftNNspecialSCAgbl ftNNlearnSCAgbl 
global ftNNer_typeSCAgbl ftNNer_betaSCAgbl ftNNconvSCAgbl 
global ftNNno_inSCAgbl ftNNno_hidSCAgbl ftNNno_outSCAgbl ftNNno_totSCAgbl
global ftNNno_in_userSCAgbl ftNNh_layerSCAgbl ftNNno_hidVECgbl 
global ftNNassocMATgbl ftNNassnCNTgbl ftNNassocOPTgbl
global ftNNweightMATgbl ftNNweightOPTgbl
global ftNNn_iparmVECgbl ftNNn_hparmVECgbl ftNNn_oparmVECgbl
global ftNNn_imaxVECgbl ftNNn_iminVECgbl ftNNn_itypeVECgbl ftNNn_inameVECgbl
global ftNNn_hmaxVECgbl ftNNn_hminVECgbl ftNNn_htypeVECgbl ftNNn_hnameVECgbl
global ftNNn_omaxVECgbl ftNNn_ominVECgbl ftNNn_otypeVECgbl ftNNn_onameVECgbl
global ftNNbase_maxSCAgbl ftNNbase_minSCAgbl ftNNbase_resSCAgbl
global ftNNn_iparmVECgbl ftNNn_hparmVECgbl ftNNn_oparmVECgbl
global ftNNv_stringMATgbl ftNNv_countSCAgbl
global ftNNt_errorVECgbl ftNNv_errorVECgbl
global ftNNrbf_meanMATgbl ftNNrbf_sdMATgbl 

global ftNNfn_errMATgbl ftNNer_typeSCAgbl ftNNbetaSCAgbl
global ftNNno_inSCAgbl ftNNno_hidSCAgbl ftNNno_outSCAgbl ftNNno_hidVECgbl
global ftNNweightMATgbl ftNNer_betaSCAgbl ftNNv_pcntSCAgbl
global ftNNn_epSCAgbl ftNNe_thSCAgbl ftNNlrSCAgbl ftNNmrSCAgbl
global ftNNv_pcntSCAgbl ftNNfn_actMATgbl ftNNlearnSCAgbl
global ftNNsimFLGgbl ftNNtrainFLGgbl ftNNcnvSTRgbl

global ftNNn_inameVECgbl ftNNn_imaxVECgbl ftNNn_iminVECgbl ftNNn_itypeVECgbl
global ftNNn_hnameVECgbl ftNNn_hmaxVECgbl ftNNn_hminVECgbl ftNNn_htypeVECgbl
global ftNNn_onameVECgbl ftNNn_omaxVECgbl ftNNn_ominVECgbl ftNNn_otypeVECgbl

global ftNNbase_maxSCAgbl ftNNbase_minSCAgbl ftNNbase_resSCAgbl ftNNact_parmMATgbl

global ftNNn_iparmVECgbl ftNNn_hparmVECgbl ftNNn_oparmVECgbl
global ftNNtrainFLGgbl ftNNsimFLGgbl ftNNst_flagSCAgbl
global ftNNv_stringMATgbl ftNNv_countSCAgbl ftNNpl_freqSCAgbl
global ftNNoldfile

if sum(ftNNoldfile)~=sum(ftNNfile)
   load ft3nn_def
   eval(['load ' ftNNfile]);
end

ftNNoldfile=ftNNfile;

ftNNtrainFLGgbl=0; 
ftNNsimFLGgbl=1; 

ni_=ftNNno_inSCAgbl;
nh_=ftNNno_hidSCAgbl;
no_=ftNNno_outSCAgbl;

neuron_id_=[ftNNn_itypeVECgbl(1:ni_,:); 
            ftNNn_htypeVECgbl(1:nh_,:);
            ftNNn_otypeVECgbl(1:no_,:)];

neuron_parm_=[ftNNn_iparmVECgbl(1:ni_,:);
              ftNNn_hparmVECgbl(1:nh_,:);
              ftNNn_oparmVECgbl(1:no_,:)];

nn_desired = 0;

v_error=0;
t_error=0;

nn_in=nn_in_mat;
if     ftNNtypeSCAgbl==1 %BP Fully connected
   nn_in_=[ones(size(nn_in,1),1) nn_in];
   [nn_out,t_error,v_error]=ftnnbp(nn_in_,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==2 %BP Recurrent
   nn_in_=[ones(size(nn_in,1),1) nn_in];
   [nn_out,t_error,v_error]=ftnnbr(nn_in_,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==3 %BP Layers
   nn_in_=[ones(size(nn_in,1),1) nn_in];
   [nn_out,t_error,v_error]=ftnnbl(nn_in_,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==4 %Fast BP

elseif ftNNtypeSCAgbl==5 %Linear 
   neuron_parm_=ftNNn_oparmVECgbl(1:no_,:);
   [nn_out,t_error,v_error]=ftnnln(nn_in,nn_desired,neuron_parm_);

elseif ftNNtypeSCAgbl==6 %Perceptrons
   [nn_out,t_error,v_error]=ftnnpc(nn_in,nn_desired);

elseif ftNNtypeSCAgbl==7 %Hebbian
   [nn_out,t_error,v_error]=ftnnhb(nn_in,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==8 %Instar
   [nn_out,t_error,v_error]=ftnnil(nn_in,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==9 %Outstar
   [nn_out,t_error,v_error]=ftnnol(nn_in,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==10 %Kohonen
   [nn_out,t_error,v_error]=ftnnkn(nn_in,nn_desired);

elseif ftNNtypeSCAgbl==11 %Counter Prop
   [nn_out,t_error,v_error]=ftnncp(nn_in,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==12 %LVQ
   [nn_out,t_error,v_error]=ftnnlv(nn_in,nn_desired,neuron_id_,neuron_parm_);

elseif ftNNtypeSCAgbl==13 %ART

elseif ftNNtypeSCAgbl==14 %RBF
   [nn_out,t_error]=ftnnrb(nn_in,nn_desired);
end
nn_out_mat=nn_out;



%===================================================================
% end ftnnsim.m
%
