%
% ftusecal.m
%
% =========================================================================  
%                          FlexTools(FS) for Matlab                          
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)       	             
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 06/05/1995                                  
% =========================================================================  
%
%
% Mapping provided from user interface to the fuzzy inference engine
% FlexTool: USEr data structure to CALculate data structure
% ftusecal!
% Necessary since user interface uses static data structures
% whereas the inference engine requires dynamic data structures.
%
% This function has to be run every time a fuzzy system is created
% or modified (ie the rulebase, and other relevant selections!)
%

function outs=ftusecal

%
% GLOBAL
%

global ftFSaNo_lingCNTgbl ftFSaLVidxVECgbl ftFSaoLVidxVECgbl ftFSmax_LVSCAgbl
global ftFSaNo_critCNTgbl ftFSNo_inCNTgbl ftFSNo_outCNTgbl 
global ftFSilingVECgbl ftFSolingVECgbl 
global ftFSidiffVECgbl ftFSodiffVECgbl ftFSiminVECgbl ftFSominVECgbl ftFSimaxVECgbl ftFSomaxVECgbl
global ftFSiLVtypeVECgbl ftFSoLVtypeVECgbl ftFSiLVlearnVECgbl ftFSoLVlearnVECgbl
global ftFSbase_maxSCAgbl ftFSbase_minSCAgbl ftFSbase_diffSCAgbl
global ftFSNo_ilingCNTgbl ftFSNo_olingCNTgbl ftFSatot_lingVECgbl 
global ftFSincMATgbl ftFSinrMATgbl ftFSinlMATgbl
global ftFSoutcMATgbl ftFSoutrMATgbl ftFSoutlMATgbl
global ftFSaincMATgbl ftFSainrMATgbl ftFSainlMATgbl
global ftFSaoutcMATgbl ftFSaoutrMATgbl ftFSaoutlMATgbl
global ftFSiparmMATgbl ftFSoparmMATgbl ftFSparmMATgbl
global ftFSNo_rulesCNTgbl ftioLVtypeVECgbl ftioLVlearnVECgbl
global ftFSmax_inSCAgbl ftFSmax_outSCAgbl ftFSmax_LVSCAgbl
global ftFSsum_lingVECgbl


outs=1;

% 
% total Number of criteria = Number of inputs + Number of outputs
%
ftFSaNo_critCNTgbl = ftFSNo_inCNTgbl + ftFSNo_outCNTgbl;

%
% Convert the static input and output linguistic variable Numbers
% (how many for each variable) into a dynamic sequential structure,
% ftFSaNo_lingCNTgbl(1:ftFSaNo_critCNTgbl)
%
ftFSaNo_lingCNTgbl = [ ftFSilingVECgbl(1:ftFSNo_inCNTgbl) ftFSolingVECgbl(1:ftFSNo_outCNTgbl) ];


% 
% Total Number of Linguistic Variables = Number of Input + Output Variables
%
ftFSatot_lingVECgbl = ftFSNo_ilingCNTgbl + ftFSNo_olingCNTgbl;

% 
% find the difference (distance of) between the min and max of the
% universe of discourse
%
ftFSidiffVECgbl = ftFSimaxVECgbl-ftFSiminVECgbl;
ftFSodiffVECgbl = ftFSomaxVECgbl-ftFSominVECgbl;

% 
% find the difference (distance of) between the min and max of the
% base set
%
ftFSbase_diffSCAgbl = ftFSbase_maxSCAgbl - ftFSbase_minSCAgbl;

%
% generate ftioLVtypeVECgbl ftioLVlearnVECgbl
%
ftioLVtypeVECgbl=[];
ftioLVlearnVECgbl=[];
ftFSparmMATgbl=[];
strtidx=1;
strtidx2=1;
for(i=1:ftFSNo_inCNTgbl)
	endidx       = strtidx+ftFSilingVECgbl(i)-1;
	endidx2      = strtidx2+ftFSilingVECgbl(i)-1;
	ftioLVtypeVECgbl  = [ftioLVtypeVECgbl ftFSiLVtypeVECgbl(strtidx2:endidx2)];
	ftioLVlearnVECgbl = [ftioLVlearnVECgbl ftFSiLVlearnVECgbl(strtidx2:endidx2)];

	for(ii=1:4)
	   cc=1;
	   for(jj=strtidx2:endidx2)
		if((ftFSiLVtypeVECgbl(jj)==3)&(ii==2)) % if FMF type is gaussian
			passtype_ = 5;
		elseif((ftFSiLVtypeVECgbl(jj)==4)&(ii==2))  % if FMF type is generalized bell
			passtype_ = 7;
		elseif((ftFSiLVtypeVECgbl(jj)==5)&(ii==3))  % if FMF type is exponential
			passtype_ = 8;
		else    
			passtype_ = 3;
		end;
	               temp_(cc,ii)=ftmaptob(ftFSiparmMATgbl(jj,ii),passtype_,i);		
		cc=cc+1;
	   end; % for
	end; % for ii loop
        ftFSparmMATgbl    = [ftFSparmMATgbl; temp_];
        clear temp_

	strtidx      = strtidx+ftFSmax_inSCAgbl;
	strtidx2     = strtidx2 + ftFSmax_LVSCAgbl;
end; % for i loop

strtidx=1;
strtidx2=1;
for(i=1:ftFSNo_outCNTgbl)
	endidx       = strtidx+ftFSolingVECgbl(i)-1;
	endidx2	     = strtidx2+ftFSolingVECgbl(i)-1;
	ftioLVtypeVECgbl  = [ftioLVtypeVECgbl ftFSoLVtypeVECgbl(strtidx2:endidx2)];
	ftioLVlearnVECgbl = [ftioLVlearnVECgbl ftFSoLVlearnVECgbl(strtidx2:endidx2)];

	for(ii=1:4)
	   cc=1;
	   for(jj=strtidx2:endidx2)
		if((ftFSoLVtypeVECgbl(jj)==3)&(ii==2))      % if FMF type is gaussian
			passtype_ = 6;
		elseif((ftFSoLVtypeVECgbl(jj)==4)&(ii==2))  % if FMF type is generalized bell
			passtype_ = 7;
		elseif((ftFSoLVtypeVECgbl(jj)==5)&(ii==3))  % if FMF type is exponential
			passtype_ = 8;
		else    
			passtype_ = 4;
		end; %if
	              temp_(cc,ii)=ftmaptob(ftFSoparmMATgbl(jj,ii),passtype_,i);
		cc=cc+1;
	   end; %for jj loop	
        end; % for ii loop
	ftFSparmMATgbl    = [ftFSparmMATgbl; temp_];
        clear temp_

	strtidx      = strtidx+ftFSmax_outSCAgbl;
	strtidx2     = strtidx2+ftFSmax_LVSCAgbl;
end; % for i loop

%
% map the linguistic variable index from the static data structure
% into ftFSaLVidxVECgbl(1:ftFSatot_lingVECgbl) in a sequential form
%
% This mapping holds good for mapping back to
%	ftFSiLVlearnVECgbl
%	ftFSoLVlearnVECgbl
%	ftFSiLVtypeVECgbl
%	ftFSoLVtypeVECgbl
%
% Note: Only a subset of these will be used for efm! 
%

k = 1;
for i=1:ftFSaNo_critCNTgbl
	for j=1:ftFSaNo_lingCNTgbl(i)
		ftFSaLVidxVECgbl(k) = ((i-1)*ftFSmax_LVSCAgbl)+j;
		k = k + 1;
	end; % for j loop
end; % for i loop


%----------------------------------------------------------------
% determine the linguistic variable whose membership function has
% the maximum number of parameters. mat_parm matrix will have
% max_fmfp number of columns.
%----------------------------------------------------------------
%	max_fmfp = max(max(No_fmfp));
	
%-------------------------------------------------------------
% ftFSNo_lingCNTgbl vector has information about the number of linguistc 
% variables the user has choosen for each criteria.
% We will now create another vector ftFSsum_lingVECgbl that contains
% the cumulative sum of the number of linguistic variables.
%-------------------------------------------------------------
	ftFSsum_lingVECgbl(1) = 0;
	for(i=2:ftFSNo_inCNTgbl)
		ftFSsum_lingVECgbl(i)	= ftFSsum_lingVECgbl(i-1) + ftFSilingVECgbl(i-1);
	end % for i loop
	for(i=1:ftFSNo_outCNTgbl)
		ftFSsum_lingVECgbl(ftFSNo_inCNTgbl+i)= ftFSsum_lingVECgbl(ftFSNo_inCNTgbl+i-1) + ftFSolingVECgbl(i);
	end % for i loop

%-----------------------------------------------
% Create a mapping for No_fmfp such that
% the number of fmfp is stored with reference to
% a serial count of the linguistic variable
%-----------------------------------------------
 k=1;	
 for(i=1:ftFSaNo_critCNTgbl)
 	for(j=1:ftFSaNo_lingCNTgbl(i))
 		l_rmap(k) = i;
 		l_cmap(k) = j;
 		k = k + 1;
 	end % for j loop
 end % for i loop
 		
% 
% PACKING MATRICES
%
% ftFSincMATgbl, ftFSinrMATgbl, ftFSinlMATgbl, 
% ftFSoutcMATgbl, ftFSoutrMATgbl, ftFSoutlMATgbl
%
% Currently are static! ie, they are build with max. values
% and the entries are maintained w.r.t. the max. 
%
% Need to change them to dynamic form!
% for example, for a particular ftFSrule_ you may be using only criteria
% 2, and 4 out of the possible 6. In ftFSincMATgbl, the entries would be
% 2 for column 2 and 4 for column 4, where as the rest would be 0.
%
% for the inference engine all that is needed is a matrix that 
% specifies criteria as 2 in column 1 and 4 in column 2, the rest being
% zeros!
% subracting by 1 since, when we created ftFSincMATgbl, we set it as
% default=1 which is for no selection (according to the menu for
% selecting the variable and linguistic term). 

[c,r]=find(ftFSincMATgbl' > 1);
if(r)
	ftFSaincMATgbl = ftmatpak(ftFSincMATgbl-1,r,c,ftFSNo_rulesCNTgbl,ftFSNo_inCNTgbl,0);
	ftFSainrMATgbl = ftmatpak(ftFSinrMATgbl,r,c,ftFSNo_rulesCNTgbl,ftFSNo_inCNTgbl,0);
	ftFSainlMATgbl = ftmatpak(ftFSinlMATgbl,r,c,ftFSNo_rulesCNTgbl,ftFSNo_inCNTgbl,0);

	[c,r]=find(ftFSoutcMATgbl' > 1);
	if(r)
		ftFSaoutcMATgbl = ftmatpak(ftFSoutcMATgbl-1,r,c,ftFSNo_rulesCNTgbl,ftFSNo_outCNTgbl,0);
		ftFSaoutrMATgbl = ftmatpak(ftFSoutrMATgbl,r,c,ftFSNo_rulesCNTgbl,ftFSNo_outCNTgbl,0);
		ftFSaoutlMATgbl = ftmatpak(ftFSoutlMATgbl,r,c,ftFSNo_rulesCNTgbl,ftFSNo_outCNTgbl,0);
	else
		ftdispwn('Attention! Exception!!' , 'Sorry! No Outputs Selected!! First Build FS');
		outs=0;
	end; % if
else
	ftdispwn('Attention! Exception!!' , 'Sorry! No Inputs Selected!! First Build FS');
	outs=0;
end; % if

%
% end ftusecal.m
%  


